# encoding: utf-8
"""
Represents a connection to the ECS service.
"""

import warnings

from footmark.exception import FootmarkClientError
from footmark.connection import ACSQueryConnection
from footmark.ecs.instance import Instance
from footmark.ecs.regioninfo import RegionInfo
from footmark.exception import ECSResponseError
import six

class ECSConnection(ACSQueryConnection):


    # SDKVersion = footmark.config.get('Footmark', 'ecs_version', '2014-05-26')
    SDKVersion = '2014-05-26'
    DefaultRegionId = 'cn-hangzhou'
    DefaultRegionName = u'杭州'.encode("UTF-8")
    ResponseError = ECSResponseError


    def __init__(self, acs_access_key_id=None, acs_secret_access_key=None,
                 region=None, sdk_version= None, security_token=None,):
        """
        Init method to create a new connection to ECS.
        """
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionId)
        self.region = region
        if sdk_version:
            self.SDKVersion = sdk_version

        self.ECSSDK = 'aliyunsdkecs.request.v' + self.SDKVersion.replace('-','')

        super(ECSConnection, self).__init__(acs_access_key_id,
                                            acs_secret_access_key,
                                            self.region, self.ECSSDK, security_token)

    def build_filter_params(self, params, filters):
        if not isinstance(filters, dict):
            return

        flag = 1
        for key,value in filters.items():
            acs_key = key
            if acs_key.startswith('tag:'):
                while(('set_Tag%dKey' % flag) in params ):
                    flag += 1
                if flag<6:
                    params['set_Tag%dKey' % flag] = acs_key[4:]
                    params['set_Tag%dValue' % flag] = filters[acs_key]
                flag += 1
                continue
            if key == 'group_id':
                if not value.startswith('sg-') or len(value) != 12:
                    warnings.warn(
                        "The group-id filter now requires a security group "
                        "identifier (sg-*) instead of a security group ID. "
                        "The group-id " + value + "may be invalid.",
                        UserWarning)
                params['set_SecurityGroupId'] = value
                continue
            if not isinstance(value, dict):
                acs_key = ''.join(s.capitalize() for s in acs_key.split('_'))
                params['set_' + acs_key] = value
                continue

            self.build_filters_params(params, value)

    # Instance methods

    def get_all_instances(self, instance_ids=None, filters=None, max_results=None):
        """
        Retrieve all the instance associated with your account.

        :rtype: list
        :return: A list of  :class:`footmark.ecs.instance.Reservation`

        """
        warnings.warn(('The current get_all_instances implementation will be '
                       'replaced with get_all_reservations.'),
                      PendingDeprecationWarning)

        params = {}
        if instance_ids:
            self.build_list_params(params, instance_ids, 'InstanceIds')
        if filters:
            self.build_filter_params(params, filters)
        if max_results is not None:
            params['MaxResults'] = max_results
        return self.get_list('DescribeInstances', params, ['Instances', Instance])

    def start_instances(self, instance_ids=None):
        """
        Start the instances specified

        :type instance_ids: list
        :param instance_ids: A list of strings of the Instance IDs to start

        :rtype: list
        :return: A list of the instances started
        """
        params = {}
        results = []
        if instance_ids:
            if isinstance(instance_ids, six.string_types):
                instance_ids = [instance_ids]
            for instance_id in instance_ids:
                self.build_list_params(params, instance_id, 'InstanceId')
                if self.get_status('StartInstance', params):
                    results.append(instance_id)
        return results

    def stop_instances(self, instance_ids=None, force=False):
        """
        Stop the instances specified

        :type instance_ids: list
        :param instance_ids: A list of strings of the Instance IDs to stop

        :type force: bool
        :param force: Forces the instance to stop

        :rtype: list
        :return: A list of the instances stopped
        """
        params = {}
        results = []
        if force:
            self.build_list_params(params, 'true', 'ForceStop')
        if instance_ids:
            if isinstance(instance_ids, six.string_types):
                instance_ids = [instance_ids]
            for instance_id in instance_ids:
                self.build_list_params(params, instance_id, 'InstanceId')
                if self.get_status('StopInstance', params):
                    results.append(instance_id)
        return results

    def reboot_instances(self, instance_ids=None, force=False):
        """
        Reboot the specified instances.

        :type instance_ids: list
        :param instance_ids: The instances to terminate and reboot

        :type force: bool
        :param force: Forces the instance to stop

        """
        params = {}
        results = []
        if force:
            self.build_list_params(params, 'true', 'ForceStop')
        if instance_ids:
            if isinstance(instance_ids, six.string_types):
                instance_ids = [instance_ids]
            for instance_id in instance_ids:
                self.build_list_params(params, instance_id, 'InstanceId')
                if self.get_status('RebootInstance', params):
                    results.append(instance_id)

        return results

    def terminate_instances(self, instance_ids=None, force=False):
        """
        Terminate the instances specified

        :type instance_ids: list
        :param instance_ids: A list of strings of the Instance IDs to terminate

        :type force: bool
        :param force: Forces the instance to stop

        :rtype: list
        :return: A list of the instance_ids terminated
        """
        params = {}
        result = []
        if force:
            self.build_list_params(params, 'true', 'ForceStop')
        if instance_ids:
            if isinstance(instance_ids, six.string_types):
                instance_ids = [instance_ids]
            for instance_id in instance_ids:
                self.build_list_params(params, instance_id, 'InstanceId')
                if self.get_status('DeleteInstance', params):
                    result.append(instance_id)
        return result



"""
Represents an ECS Instance
"""
import json
from footmark.ecs.ecsobject import *

class Instance(TaggedECSObject):
    """
    Represents an instance.
    """

    def __init__(self, connection=None):
        super(Instance, self).__init__(connection)
        self.tags = {}


    def __repr__(self):
        return 'Instance:%s' % self.id

    def __getattr__(self, name):
        if name == 'id':
            return self.instance_id
        if name == 'state':
            return self.status
        if name in ('private_ip', 'inner_ip', 'inner_ip_address'):
            return self.inner_ip_address
        if name in ('public_ip', 'assign_public_ip', 'ip_address'):
            return self.public_ip_address
        if name == 'vpc_private_ip':
            return self.private_ip_address
        if name in ('vpc_vswitch_id', 'vswitch_id', 'vpc_subnet_id', 'subnet_id'):
            return self.v_switch_id
        if name == 'eip' and self.eip_address:
            return self.eip_address.get('ip_address', None)
        raise AttributeError

    def __setattr__(self, name, value):
        if name == 'id':
            self.instance_id = value
        if name == 'status':
            value = value.lower()
        if name == 'state':
            self.status = value
        if name in ('public_ip_address', 'inner_ip_address', 'private_ip_address'):
            if isinstance(value, dict) and value['ip_address']:
                value = value['ip_address'][0]
        if name in ('private_ip', 'inner_ip'):
            self.inner_ip_address = value
        if name in ('public_ip', 'assign_public_ip'):
            self.public_ip_address = value
        if name == 'vpc_private_ip':
            self.private_ip_address = value
        if name in ('vpc_vswitch_id', 'vswitch_id', 'vpc_subnet_id', 'subnet_id'):
            self.v_switch_id = value
        if name == 'eip' and self.eip_address:
            self.eip_address['ip_address'] = value
        if name == 'tags' and value:
            v = {}
            for tag in value['tag']:
                v[tag.get('TagKey')] = tag.get('TagValue', None)
            value = v
        super(TaggedECSObject, self).__setattr__(name, value)

    def _update(self, updated):
        self.__dict__.update(updated.__dict__)

    def update(self, validate=False):
        """
        Update the instance's state information by making a call to fetch
        the current instance attributes from the service.

        :type validate: bool
        :param validate: By default, if ECS returns no data about the
                         instance the update method returns quietly.  If
                         the validate param is True, however, it will
                         raise a ValueError exception if no data is
                         returned from ECS.
        """
        rs = self.connection.get_all_instances([self.id])
        if len(rs) > 0:
            for r in rs:
                if r.id == self.id:
                    self._update(r)
        elif validate:
            raise ValueError('%s is not a valid Instance ID' % self.id)
        return self.state

    def start(self):
        """
        Start the instance.
        """
        rs = self.connection.start_instances([self.id])
        # if len(rs) > 0:
        #     self._update(rs[0])

    def stop(self, force=False):
        """
        Stop the instance

        :type force: bool
        :param force: Forces the instance to stop

        :rtype: list
        :return: A list of the instances stopped
        """
        rs = self.connection.stop_instances([self.id], force)
        # if len(rs) > 0:
        #     self._update(rs[0])

    def reboot(self, force=False):
        """
        Restart the instance.

        :type force: bool
        :param force: Forces the instance to stop
        """
        return self.connection.reboot_instances([self.id], force)

    def terminate(self, force=False):
        """
        Terminate the instance

        :type force: bool
        :param force: Forces the instance to terminate
        """
        rs = self.connection.terminate_instances([self.id], force)
        # if len(rs) > 0:
        #     self._update(rs[0])



#
from footmark.pyami.config import Config, FootmarkConfigLocations
import datetime
import os
import logging
import logging.config


__version__ = '1.0.5'
Version = __version__  # for backware compatibility

datetime.datetime.strptime('', '')

config = Config()


def init_logging():
    for file in FootmarkConfigLocations:
        try:
            logging.config.fileConfig(os.path.expanduser(file))
        except:
            pass


class NullHandler(logging.Handler):
    def emit(self, record):
        pass

log = logging.getLogger('footmark')
perflog = logging.getLogger('footmark.perf')
log.addHandler(NullHandler())
perflog.addHandler(NullHandler())
init_logging()

# convenience function to set logging to a particular file

def set_file_logger(name, filepath, level=logging.INFO, format_string=None):
    global log
    if not format_string:
        format_string = "%(asctime)s %(name)s [%(levelname)s]:%(message)s"
    logger = logging.getLogger(name)
    logger.setLevel(level)
    fh = logging.FileHandler(filepath)
    fh.setLevel(level)
    formatter = logging.Formatter(format_string)
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    log = logger


def set_stream_logger(name, level=logging.DEBUG, format_string=None):
    global log
    if not format_string:
        format_string = "%(asctime)s %(name)s [%(levelname)s]:%(message)s"
    logger = logging.getLogger(name)
    logger.setLevel(level)
    fh = logging.StreamHandler()
    fh.setLevel(level)
    formatter = logging.Formatter(format_string)
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    log = logger


def connect_ecs(acs_access_key_id=None, acs_secret_access_key=None, **kwargs):
    """
    :type acs_access_key_id: string
    :param acs_access_key_id: Your AWS Access Key ID

    :type acs_secret_access_key: string
    :param acs_secret_access_key: Your AWS Secret Access Key

    :rtype: :class:`footmark.ecs.connection.ECSConnection`
    :return: A connection to Amazon's ECS
    """
    from footmark.ecs.connection import ECSConnection
    return ECSConnection(acs_access_key_id, acs_secret_access_key, **kwargs)




"""
This class encapsulates the provider-specific header differences.
"""

class Provider(object):

    def __init__(self, name, access_key=None, secret_key=None, security_token=None):
        self.access_key = access_key
        self.secret_key = secret_key
        self.security_token = security_token
        
    def get_access_key(self):
        # if self._credentials_need_refresh():
        #     self._populate_keys_from_metadata_server()
        return self._access_key

    def set_access_key(self, value):
        self._access_key = value

    access_key = property(get_access_key, set_access_key)

    def get_secret_key(self):
        # if self._credentials_need_refresh():
        #     self._populate_keys_from_metadata_server()
        return self._secret_key

    def set_secret_key(self, value):
        self._secret_key = value

    secret_key = property(get_secret_key, set_secret_key)

    def get_security_token(self):
        if self._credentials_need_refresh():
            self._populate_keys_from_metadata_server()
        return self._security_token

    def set_security_token(self, value):
        self._security_token = value

    security_token = property(get_security_token, set_security_token)

# Static utility method for getting default Provider.
def get_default():
    return Provider('acs')




#
import os
import re
import warnings
import StringIO
import footmark

# from footmark.compat import expanduser, ConfigParser, NoOptionError, NoSectionError, StringIO
from ConfigParser import SafeConfigParser as ConfigParser
try:
    os.path.expanduser('~')
    expanduser = os.path.expanduser
except (AttributeError, ImportError):
    # This is probably running on App Engine.
    expanduser = (lambda x: x)
try:
    import simplejson as json
except ImportError:
    import json


# By default we use two locations for the footmark configurations,
# /etc/footmark.cfg and ~/.footmark (which works on Windows and Unix).
FootmarkConfigPath = '/etc/footmark.cfg'
FootmarkConfigLocations = [FootmarkConfigPath]
UserConfigPath = os.path.join(expanduser('~'), '.footmark')
FootmarkConfigLocations.append(UserConfigPath)

# If there's a FOOTMARK_CONFIG variable set, we load ONLY
# that variable
if 'FOOTMARK_CONFIG' in os.environ:
    FootmarkConfigLocations = [expanduser(os.environ['FOOTMARK_CONFIG'])]

# If there's a FOOTMARK_PATH variable set, we use anything there
# as the current configuration locations, split with os.pathsep.
elif 'FOOTMARK_PATH' in os.environ:
    FootmarkConfigLocations = []
    for path in os.environ['FOOTMARK_PATH'].split(os.pathsep):
        FootmarkConfigLocations.append(expanduser(path))

class Config(object):

    def __init__(self, path=None, fp=None, do_load=True):
        self._parser = ConfigParser({'working_dir': '/mnt/pyami',
                                     'debug': '0'})
        if do_load:
            if path:
                self.load_from_path(path)
            elif fp:
                self.readfp(fp)
            else:
                self.read(FootmarkConfigLocations)

    def __setstate__(self, state):
        # There's test that verify that (transitively) a Config
        # object can be pickled.  Now that we're storing a _parser
        # attribute and relying on __getattr__ to proxy requests,
        # we need to implement setstate to ensure we don't get
        # into recursive loops when looking up _parser when
        # this object is unpickled.
        self._parser = state['_parser']

    def __getattr__(self, name):
        return getattr(self._parser, name)

    def has_option(self, *args, **kwargs):
        return self._parser.has_option(*args, **kwargs)

    def load_from_path(self, path):
        file = open(path)
        for line in file.readlines():
            match = re.match("^#import[\s\t]*([^\s^\t]*)[\s\t]*$", line)
            if match:
                extended_file = match.group(1)
                (dir, file) = os.path.split(path)
                self.load_from_path(os.path.join(dir, extended_file))
        self.read(path)

    def save_option(self, path, section, option, value):
        """
        Write the specified Section.Option to the config file specified by path.
        Replace any previous value.  If the path doesn't exist, create it.
        Also add the option the the in-memory config.
        """
        config = ConfigParser()
        config.read(path)
        if not config.has_section(section):
            config.add_section(section)
        config.set(section, option, value)
        fp = open(path, 'w')
        config.write(fp)
        fp.close()
        if not self.has_section(section):
            self.add_section(section)
        self.set(section, option, value)

    def save_user_option(self, section, option, value):
        self.save_option(UserConfigPath, section, option, value)

    def save_system_option(self, section, option, value):
        self.save_option(FootmarkConfigPath, section, option, value)

    # def get_instance(self, name, default=None):
    #     try:
    #         val = self.get('Instance', name)
    #     except (NoOptionError, NoSectionError):
    #         val = default
    #     return val
    #
    # def get_user(self, name, default=None):
    #     try:
    #         val = self.get('User', name)
    #     except (NoOptionError, NoSectionError):
    #         val = default
    #     return val
    #
    # def getint_user(self, name, default=0):
    #     try:
    #         val = self.getint('User', name)
    #     except (NoOptionError, NoSectionError):
    #         val = default
    #     return val
    #
    # def get_value(self, section, name, default=None):
    #     return self.get(section, name, default)
    #
    # def get(self, section, name, default=None):
    #     try:
    #         return self._parser.get(section, name)
    #     except (NoOptionError, NoSectionError):
    #         return default
    #
    # def getint(self, section, name, default=0):
    #     try:
    #         return self._parser.getint(section, name)
    #     except (NoOptionError, NoSectionError):
    #         return int(default)
    #
    # def getfloat(self, section, name, default=0.0):
    #     try:
    #         return self._parser.getfloat(section, name)
    #     except (NoOptionError, NoSectionError):
    #         return float(default)

    def getbool(self, section, name, default=False):
        if self.has_option(section, name):
            val = self.get(section, name)
            if val.lower() == 'true':
                val = True
            else:
                val = False
        else:
            val = default
        return val

    def setbool(self, section, name, value):
        if value:
            self.set(section, name, 'true')
        else:
            self.set(section, name, 'false')

    # def dump(self):
    #     s = StringIO()
    #     self.write(s)
    #     print(s.getvalue())
    #
    # def dump_safe(self, fp=None):
    #     if not fp:
    #         fp = StringIO()
    #     for section in self.sections():
    #         fp.write('[%s]\n' % section)
    #         for option in self.options(section):
    #             if option == 'aws_secret_access_key':
    #                 fp.write('%s = xxxxxxxxxxxxxxxxxx\n' % option)
    #             else:
    #                 fp.write('%s = %s\n' % (option, self.get(section, option)))
    #
    # def dump_to_sdb(self, domain_name, item_name):
    #     from footmark.compat import json
    #     sdb = footmark.connect_sdb()
    #     domain = sdb.lookup(domain_name)
    #     if not domain:
    #         domain = sdb.create_domain(domain_name)
    #     item = domain.new_item(item_name)
    #     item.active = False
    #     for section in self.sections():
    #         d = {}
    #         for option in self.options(section):
    #             d[option] = self.get(section, option)
    #         item[section] = json.dumps(d)
    #     item.save()
    #
    # def load_from_sdb(self, domain_name, item_name):
    #     from footmark.compat import json
    #     sdb = footmark.connect_sdb()
    #     domain = sdb.lookup(domain_name)
    #     item = domain.get_item(item_name)
    #     for section in item.keys():
    #         if not self.has_section(section):
    #             self.add_section(section)
    #         d = json.loads(item[section])
    #         for attr_name in d.keys():
    #             attr_value = d[attr_name]
    #             if attr_value is None:
    #                 attr_value = 'None'
    #             if isinstance(attr_value, bool):
    #                 self.setbool(section, attr_name, attr_value)
    #             else:
    #                 self.set(section, attr_name, attr_value)
